@@ -1,7 +1,9 @@
 """Unit test for Parameterized."""
+import abc
 import inspect
 import re
 import unittest
+import warnings
 
 import param
 import numbergen
@@ -16,9 +18,12 @@ import random
 
 from param import parameterized, Parameter
 from param.parameterized import (
+    ParameterizedABC,
     ParamOverrides,
+    ParameterizedMetaclass,
     Undefined,
     default_label_formatter,
+    edit_constant,
     no_instance_params,
     shared_parameters,
 )
@@ -270,7 +275,7 @@ class TestParameterized(unittest.TestCase):
     def test_constant_parameter_modify_class_before(self):
         """
         Test you can set on class and the new default is picked up
-        by new instances
+        by new instances.
         """
         TestPO.const=9
         testpo = TestPO()
@@ -279,7 +284,7 @@ class TestParameterized(unittest.TestCase):
     def test_constant_parameter_modify_class_after_init(self):
         """
         Test that setting the value on the class doesn't update the instance value
-        even when the instance value hasn't yet been set
+        even when the instance value hasn't yet been set.
         """
         oobj = []
         class P(param.Parameterized):
@@ -313,6 +318,17 @@ class TestParameterized(unittest.TestCase):
         testpo = TestPO()
         self.assertEqual(testpo.const,9)
 
+    def test_edit_constant(self):
+        testpo = TestPO(const=670)
+        # Checking no parameter was already instantiated
+        assert not testpo._param__private.params
+        with edit_constant(testpo):
+            testpo.const = 891
+        assert testpo.const == 891
+        assert testpo.param['const'].constant
+        assert TestPO.param['const'].constant
+        assert TestPO.param['const'].default not in (670, 891)
+
     def test_readonly_parameter(self):
         """Test that you can't set a read-only parameter on construction or as an attribute."""
         testpo = TestPO()
@@ -365,11 +381,28 @@ class TestParameterized(unittest.TestCase):
         assert t.param['instPO'].instantiate is True
         assert isinstance(t.instPO,AnotherTestPO)
 
-    def test_abstract_class(self):
+    def test_abstract_class_attribute(self):
         """Check that a class declared abstract actually shows up as abstract."""
         self.assertEqual(TestAbstractPO.abstract, True)
         self.assertEqual(_AnotherAbstractPO.abstract, True)
         self.assertEqual(TestPO.abstract, False)
+        # Test subclasses are not abstract
+        class A(param.Parameterized):
+            __abstract = True
+        class B(A): pass
+        class C(A): pass
+        self.assertEqual(A.abstract, True)
+        self.assertEqual(B.abstract, False)
+        self.assertEqual(C.abstract, False)
+
+    def test_abstract_class_abc(self):
+        """Check that an ABC class actually shows up as abstract."""
+        class A(ParameterizedABC): pass
+        class B(A): pass
+        class C(A): pass
+        self.assertEqual(A.abstract, True)
+        self.assertEqual(B.abstract, False)
+        self.assertEqual(C.abstract, False)
 
     def test_override_class_param_validation(self):
         test = TestPOValidation()
@@ -400,7 +433,9 @@ class TestParameterized(unittest.TestCase):
                 nonlocal count
                 count += 1
 
-        p = P()
+        with warnings.catch_warnings():
+            warnings.filterwarnings('ignore', message='Setting the Parameter', category=param._utils.ParamPendingDeprecationWarning)
+            p = P()
 
         assert p.x == 1
         assert count == 0
@@ -417,7 +452,9 @@ class TestParameterized(unittest.TestCase):
                     self.sub = P(depth+1)
                 super().__init__()
 
-        p = P()
+        with warnings.catch_warnings():
+            warnings.filterwarnings('ignore', message='Setting the Parameter', category=param._utils.ParamPendingDeprecationWarning)
+            p = P()
 
         assert p.value == 'B'
         assert p.sub.value == 'B'
@@ -442,7 +479,9 @@ class TestParameterized(unittest.TestCase):
 
         # When b is instantiated the `batched` Parameter of B is set before
         # Parameterized.__init__ is called.
-        b = B()
+        with warnings.catch_warnings():
+            warnings.filterwarnings('ignore', message='Setting the Parameter', category=param._utils.ParamPendingDeprecationWarning)
+            b = B()
         assert b.batched is True
 
     def test_instantiation_param_objects_before_super_subclass(self):
@@ -536,11 +575,19 @@ class TestParameterized(unittest.TestCase):
     def test_values_name_ignored_for_instances_and_onlychanged(self):
         default_inst = param.Parameterized()
         assert 'Parameterized' in default_inst.name
-        # name ignored when automatically computed (behavior inherited from all_equal)
+        # name ignored when automatically computed (behavior inherited from is_equal)
         assert 'name' not in default_inst.param.values(onlychanged=True)
         # name not ignored when set
         assert param.Parameterized(name='foo').param.values(onlychanged=True)['name'] == 'foo'
 
+    def test_values_dyn(self):
+        # See https://github.com/holoviz/param/issues/1057
+        t = TestPO()
+        orig_default = t.param.dyn.default
+        t.param.dyn.default = 3290432424
+        values = t.param.values()
+        assert values['dyn'] == orig_default
+
     def test_param_iterator(self):
         self.assertEqual(set(TestPO.param), {'name', 'inst', 'notinst', 'const', 'dyn',
                                              'ro', 'ro2', 'ro_label', 'ro_format'})
@@ -659,6 +706,14 @@ class TestParameterized(unittest.TestCase):
         t.param._state_pop()
         assert t.param.inspect_value('dyn')==orig
 
+    def test_get_value_generator_dyn(self):
+        # See https://github.com/holoviz/param/issues/1057
+        t = TestPO()
+        orig_default = t.param.dyn.default
+        t.param.dyn.default = 9594323423
+        vg = t.param.get_value_generator('dyn')
+        assert vg == orig_default
+
     def test_label(self):
         t = TestPO()
         assert t.param['ro_label'].label == 'Ro Label'
@@ -1135,6 +1190,8 @@ def test_inheritance_default_is_None_in_sub():
 
 def test_inheritance_diamond_not_supported():
     """
+    Test that Parameters don't respect diamond inheritance.
+
     In regular Python, the value of the class attribute p on D is resolved
     to 2:
 
@@ -1274,7 +1331,7 @@ def test_inheritance_instantiate_behavior():
     assert b.param.p.instantiate is True
 
 
-def test_inheritance_constant_behavior():
+def test_inheritance_readonly_behavior():
     class A(param.Parameterized):
         p = param.Parameter(readonly=True)
 
@@ -1282,13 +1339,28 @@ def test_inheritance_constant_behavior():
         p = param.Parameter()
 
 
-    # Normally, param.Parameter(readonly=True) ends up with constant being
-    # True.
-    assert B.param.p.constant is False
+    assert B.param.p.readonly is True
+    assert B.param.p.constant is True
 
     b = B()
 
-    assert b.param.p.constant is False
+    assert b.param.p.readonly is True
+    assert b.param.p.constant is True
+
+
+def test_inheritance_constant_behavior():
+    class A(param.Parameterized):
+        p = param.Parameter(constant=True)
+
+    class B(A):
+        p = param.Parameter()
+
+
+    assert B.param.p.constant is True
+
+    b = B()
+
+    assert b.param.p.constant is True
 
 
 def test_inheritance_set_Parameter_instantiate_constant_before_instantation():
@@ -1798,3 +1870,70 @@ def test_inheritance_with_changing_class_():
     ):
         class C(B):
             p = param.ClassSelector(class_=str)
+
+
+class MyABC(ParameterizedABC):
+
+    x = param.Number()
+
+    @abc.abstractmethod
+    def method(self): pass
+
+    @property
+    @abc.abstractmethod
+    def property(self): pass
+    # Other methods like abc.abstractproperty are deprecated and can be
+    # replaced by combining @abc.abstracmethod with other decorators, like
+    # @property, @classmethod, etc. No need to test them all.
+
+
+def test_abc_insintance_metaclass():
+    assert isinstance(MyABC, ParameterizedMetaclass)
+
+
+def test_abc_param_abstract():
+    assert MyABC.abstract
+
+
+def test_abc_error_when_interface_not_implemented():
+    class Bad(MyABC):
+        def wrong_method(self): pass
+
+    with pytest.raises(TypeError, match="Can't instantiate abstract class Bad"):
+        Bad()
+
+def test_abc_basic_checks():
+    # Some very basic tests to check the concrete class works as expected.
+    class GoodConcrete(MyABC):
+        l = param.List()
+
+        def method(self):
+            return 'foo'
+
+        @property
+        def property(self):
+            return 'bar'
+
+        @param.depends('x', watch=True, on_init=True)
+        def on_x(self):
+            self.l.append(self.x)
+
+    assert issubclass(GoodConcrete, param.Parameterized)
+    assert not GoodConcrete.abstract
+
+    assert GoodConcrete.name == 'GoodConcrete'
+
+    with pytest.raises(
+        ValueError,
+        match=re.escape("Number parameter 'MyABC.x' only takes numeric values, not <class 'str'>."),
+    ):
+        GoodConcrete(x='bad')
+
+    gc = GoodConcrete(x=10)
+    assert isinstance(gc, param.Parameterized)
+    assert gc.method() == 'foo'
+    assert gc.property == 'bar'
+    assert gc.name.startswith('GoodConcrete0')
+    assert gc.l == [10]
+    gc.x += 1
+    assert gc.l == [10, 11]
